<?php

namespace Drupal\Tests\cloud\Behat\features\bootstrap;

use Drupal\DrupalExtension\Context\RawDrupalContext;

/**
 * Defines application features from the specific context.
 */
class FeatureContext extends RawDrupalContext {

  /**
   * Initializes context.
   *
   * Every scenario gets its own context instance.
   * You can also pass arbitrary arguments to the
   * context constructor through behat.yml.
   */
  public function __construct() {
  }

  /**
   * Fill in the hour/minute based on the elapsed minute from the current time.
   *
   * @var hour_id
   *  The ID of the hour list.
   * @var minute_id
   *  The ID of the minute list.
   * @var elapsed_minute
   *  The elapsed minute.
   */
  private function selectTimeByElapsedTime(string $hour_id, string $minute_id, string $elapsed_minute) {
    $element = $this->getSession()->getPage();
    $hour_select = $element->findById($hour_id);
    $minute_select = $element->findById($minute_id);

    if (empty($hour_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $hour_id, $this->getSession()->getCurrentUrl()));
    }
    if (empty($minute_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $minute_id, $this->getSession()->getCurrentUrl()));
    }

    $time = new \DateTime('now', new \DateTimeZone(date_default_timezone_get()));
    $time->add(\DateInterval::createFromDateString("$elapsed_minute minute"));
    $hour = $time->format('H');
    $minute = $time->format('i');

    $hour_select->selectOption($hour, FALSE);
    $minute_select->selectOption($minute, FALSE);
  }

  /**
   * Wait until the specified schedule time.
   *
   * @var hour_id
   *  The ID of the hour list.
   * @var minute_id
   *  The ID of the minute list.
   */
  private function waitUntilScheduleTime(string $hour_id, string $minute_id): void {
    $resource_startup_wait_minute = 1;
    $element = $this->getSession()->getPage();
    $hour_select = $element->findById($hour_id);
    $minute_select = $element->findById($minute_id);

    if (empty($hour_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $hour_id, $this->getSession()->getCurrentUrl()));
    }
    if (empty($minute_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $minute_id, $this->getSession()->getCurrentUrl()));
    }

    $hour = $hour_select->getValue();
    $minute = $minute_select->getValue();

    $now_time = new \DateTime('now', new \DateTimeZone(date_default_timezone_get()));
    $wait_until_time = new \DateTime('', new \DateTimeZone(date_default_timezone_get()));
    if (empty($hour)) {
      throw new \Exception(sprintf('Failed to get the value of %s select.', $hour_select));
    }
    if (empty($minute)) {
      throw new \Exception(sprintf('Failed to get the value of %s select.', $minute_select));
    }
    $wait_until_time->setTime($hour, $minute);
    $wait_until_time->add(\DateInterval::createFromDateString("$resource_startup_wait_minute Minutes"));
    $wait_time = $wait_until_time->diff($now_time);
    // Converts hours, minutes, and seconds to seconds.
    $wait_second = $wait_time->h * 60 * 60 + $wait_time->i * 60 + $wait_time->s;
    sleep($wait_second);
  }

  /**
   * Select the start time in elapsed minute from the current time.
   *
   * @When I select :elapsed_minutes minutes later now from the start-up time
   */
  public function selectScheduleStartupTimeByElapsedTime($elapsed_minutes) {
    $this->selectTimeByElapsedTime('edit-start-hour', 'edit-start-minute', $elapsed_minutes);
  }

  /**
   * Select the stop time in elapsed minute from the current time.
   *
   * @When I select :elapsed_minutes minutes later now from the stop time
   */
  public function selectScheduleStopTimeByElapsedTime($elapsed_minutes) {
    $this->selectTimeByElapsedTime('edit-stop-hour', 'edit-stop-minute', $elapsed_minutes);
  }

  /**
   * Wait until the stop time for Schedule.
   *
   * @When I wait until start-up time for schedule
   */
  public function waitUntilStartupTimeForSchedule(): void {
    $this->waitUntilScheduleTime('edit-start-hour', 'edit-start-minute');
  }

  /**
   * Wait until the stop time for Schedule.
   *
   * @When I wait until stop time for schedule
   */
  public function waitUntilStopTimeForSchedule(): void {
    $this->waitUntilScheduleTime('edit-stop-hour', 'edit-stop-minute');
  }

}
